/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/firehose/Firehose_EXPORTS.h>
#include <aws/firehose/model/Connectivity.h>

#include <utility>

namespace Aws {
namespace Utils {
namespace Json {
class JsonValue;
class JsonView;
}  // namespace Json
}  // namespace Utils
namespace Firehose {
namespace Model {

/**
 * <p>The authentication configuration of the Amazon MSK cluster.</p><p><h3>See
 * Also:</h3>   <a
 * href="http://docs.aws.amazon.com/goto/WebAPI/firehose-2015-08-04/AuthenticationConfiguration">AWS
 * API Reference</a></p>
 */
class AuthenticationConfiguration {
 public:
  AWS_FIREHOSE_API AuthenticationConfiguration() = default;
  AWS_FIREHOSE_API AuthenticationConfiguration(Aws::Utils::Json::JsonView jsonValue);
  AWS_FIREHOSE_API AuthenticationConfiguration& operator=(Aws::Utils::Json::JsonView jsonValue);
  AWS_FIREHOSE_API Aws::Utils::Json::JsonValue Jsonize() const;

  ///@{
  /**
   * <p>The ARN of the role used to access the Amazon MSK cluster.</p>
   */
  inline const Aws::String& GetRoleARN() const { return m_roleARN; }
  inline bool RoleARNHasBeenSet() const { return m_roleARNHasBeenSet; }
  template <typename RoleARNT = Aws::String>
  void SetRoleARN(RoleARNT&& value) {
    m_roleARNHasBeenSet = true;
    m_roleARN = std::forward<RoleARNT>(value);
  }
  template <typename RoleARNT = Aws::String>
  AuthenticationConfiguration& WithRoleARN(RoleARNT&& value) {
    SetRoleARN(std::forward<RoleARNT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The type of connectivity used to access the Amazon MSK cluster.</p>
   */
  inline Connectivity GetConnectivity() const { return m_connectivity; }
  inline bool ConnectivityHasBeenSet() const { return m_connectivityHasBeenSet; }
  inline void SetConnectivity(Connectivity value) {
    m_connectivityHasBeenSet = true;
    m_connectivity = value;
  }
  inline AuthenticationConfiguration& WithConnectivity(Connectivity value) {
    SetConnectivity(value);
    return *this;
  }
  ///@}
 private:
  Aws::String m_roleARN;

  Connectivity m_connectivity{Connectivity::NOT_SET};
  bool m_roleARNHasBeenSet = false;
  bool m_connectivityHasBeenSet = false;
};

}  // namespace Model
}  // namespace Firehose
}  // namespace Aws
